@@ -21,6 +21,7 @@ from dateutil import parser
 
 from sushy import exceptions
 from sushy.resources import constants as res_cons
+from sushy.resources.registry import attribute_registry
 from sushy.resources.registry import message_registry
 from sushy.resources import settings
 from sushy.resources.system import bios
@@ -42,16 +43,28 @@ class BiosTestCase(base.TestCase):
             self.bios_settings_json,
             self.bios_settings_json]
 
+        registries = {}
         conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/message_registry.json') as f:
             conn.get.return_value.json.return_value = json.load(f)
-            registry = message_registry.MessageRegistry(
+            msg_reg = message_registry.MessageRegistry(
                 conn, '/redfish/v1/Registries/Test',
                 redfish_version='1.0.2')
 
+            registries['Test.1.0'] = msg_reg
+
+        with open('sushy/tests/unit/json_samples/'
+                  'bios_attribute_registry.json') as f:
+            conn.get.return_value.json.return_value = json.load(f)
+            self.bios_reg = attribute_registry.AttributeRegistry(
+                conn, '/redfish/v1/Registries/BiosRegistryTest',
+                redfish_version='1.0.2')
+
+            registries['BiosRegistry.1.0'] = self.bios_reg
+
         self.sys_bios = bios.Bios(
             self.conn, '/redfish/v1/Systems/437XR1138R2/BIOS',
-            registries={'Test.1.0': registry},
+            registries=registries,
             redfish_version='1.0.2')
 
     def test__parse_attributes(self):
@@ -66,8 +79,8 @@ class BiosTestCase(base.TestCase):
         self.assertEqual('', self.sys_bios.attributes['AdminPhone'])
         self.assertEqual('Uefi', self.sys_bios.attributes['BootMode'])
         self.assertEqual(0, self.sys_bios.attributes['ProcCoreDisable'])
-        self.assertEqual([res_cons.APPLY_TIME_ON_RESET,
-                          res_cons.APPLY_TIME_MAINT_RESET],
+        self.assertEqual([res_cons.ApplyTime.ON_RESET,
+                          res_cons.ApplyTime.IN_MAINTENANCE_WINDOW_ON_RESET],
                          self.sys_bios.supported_apply_times)
         self.assertEqual(600, self.sys_bios.maintenance_window
                          .maintenance_window_duration_in_seconds)
@@ -113,7 +126,7 @@ class BiosTestCase(base.TestCase):
 
     def test_set_attribute_apply_time(self):
         self.sys_bios.set_attribute('ProcTurboMode', 'Disabled',
-                                    res_cons.APPLY_TIME_ON_RESET)
+                                    res_cons.ApplyTime.ON_RESET)
         self.sys_bios._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
             data={'Attributes': {'ProcTurboMode': 'Disabled'},
@@ -122,10 +135,11 @@ class BiosTestCase(base.TestCase):
                       'ApplyTime': 'OnReset'}})
 
     def test_set_attribute_apply_time_with_maintenance_window(self):
-        self.sys_bios.set_attribute('ProcTurboMode', 'Disabled',
-                                    res_cons.APPLY_TIME_MAINT_RESET,
-                                    datetime.datetime(2020, 9, 1, 4, 30, 0),
-                                    600)
+        self.sys_bios.set_attribute(
+            'ProcTurboMode', 'Disabled',
+            res_cons.ApplyTime.IN_MAINTENANCE_WINDOW_ON_RESET,
+            datetime.datetime(2020, 9, 1, 4, 30, 0),
+            600)
         self.sys_bios._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
             data={'Attributes': {'ProcTurboMode': 'Disabled'},
@@ -162,7 +176,7 @@ class BiosTestCase(base.TestCase):
     def test_set_attributes_apply_time(self):
         self.sys_bios.set_attributes({'ProcTurboMode': 'Disabled',
                                       'UsbControl': 'UsbDisabled'},
-                                     res_cons.APPLY_TIME_IMMEDIATE)
+                                     res_cons.ApplyTime.IMMEDIATE)
         self.sys_bios._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
             data={'Attributes': {'ProcTurboMode': 'Disabled',
@@ -172,11 +186,11 @@ class BiosTestCase(base.TestCase):
                       'ApplyTime': 'Immediate'}})
 
     def test_set_attributes_apply_time_with_maintenance_window(self):
-        self.sys_bios.set_attributes({'ProcTurboMode': 'Disabled',
-                                      'UsbControl': 'UsbDisabled'},
-                                     res_cons.APPLY_TIME_MAINT_START,
-                                     datetime.datetime(2020, 9, 1, 4, 30, 0),
-                                     600)
+        self.sys_bios.set_attributes(
+            {'ProcTurboMode': 'Disabled', 'UsbControl': 'UsbDisabled'},
+            res_cons.ApplyTime.AT_MAINTENANCE_WINDOW_START,
+            datetime.datetime(2020, 9, 1, 4, 30, 0),
+            600)
         self.sys_bios._conn.patch.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
             data={'Attributes': {'ProcTurboMode': 'Disabled',
@@ -200,7 +214,7 @@ class BiosTestCase(base.TestCase):
                           self.sys_bios.set_attributes,
                           {'ProcTurboMode': 'Disabled',
                            'UsbControl': 'UsbDisabled'},
-                          res_cons.APPLY_TIME_MAINT_START,
+                          res_cons.ApplyTime.AT_MAINTENANCE_WINDOW_START,
                           maint_window_duration=600)
 
     def test_set_attributes_maint_window_duration_missing(self):
@@ -208,7 +222,7 @@ class BiosTestCase(base.TestCase):
                           self.sys_bios.set_attributes,
                           {'ProcTurboMode': 'Disabled',
                            'UsbControl': 'UsbDisabled'},
-                          res_cons.APPLY_TIME_MAINT_START,
+                          res_cons.ApplyTime.AT_MAINTENANCE_WINDOW_START,
                           datetime.datetime.now())
 
     def test_set_attributes_on_refresh(self):
@@ -333,3 +347,57 @@ class BiosTestCase(base.TestCase):
             data={'OldPassword': 'oldpassword',
                   'NewPassword': 'newpassword',
                   'PasswordName': 'adminpassword'})
+
+    def test_get_attribute_registry(self):
+
+        registry = self.sys_bios.get_attribute_registry()
+        self.assertEqual(registry.name, 'BIOS Attribute Registry')
+        self.assertEqual(registry.description, 'This is a test of BIOS '
+                         'Attribute Registry')
+        self.assertEqual(registry.registry_entries.attributes[0].name,
+                         'SystemModelName')
+        self.assertEqual(registry.registry_entries.attributes[0].display_name,
+                         'System Model Name')
+        self.assertEqual(registry.registry_entries.attributes[0].immutable,
+                         True)
+        self.assertEqual(registry.registry_entries.attributes[0].read_only,
+                         True)
+        self.assertEqual(registry.registry_entries.attributes[0].
+                         attribute_type, 'String')
+        self.assertEqual(registry.registry_entries.attributes[1].name,
+                         'ProcVirtualization')
+        self.assertEqual(registry.registry_entries.attributes[1].display_name,
+                         'Virtualization Technology')
+        self.assertEqual(registry.registry_entries.attributes[1].immutable,
+                         False)
+        self.assertEqual(registry.registry_entries.attributes[1].read_only,
+                         False)
+        self.assertEqual(registry.registry_entries.attributes[1].
+                         reset_required, True)
+        self.assertEqual(registry.registry_entries.attributes[1].
+                         attribute_type, 'Enumeration')
+        self.assertEqual(registry.registry_entries.attributes[1].
+                         allowable_values,
+                         [{'ValueDisplayName': 'Enabled',
+                           'ValueName': 'Enabled'},
+                          {'ValueDisplayName': 'Disabled',
+                           'ValueName': 'Disabled'}])
+
+    def test_get_attribute_registry_by_key(self):
+        # This is seen on some SuperMicro hardware
+        self.sys_bios._registries[self.bios_reg.identity] = self.bios_reg
+        self.bios_reg.identity = 'I.dont.match'
+        registry = self.sys_bios.get_attribute_registry()
+        self.assertIsNotNone(registry)
+        self.assertEqual(registry.name, 'BIOS Attribute Registry')
+
+    def test_get_attribute_registry_no_lang(self):
+
+        registry = self.sys_bios.get_attribute_registry(language='zh')
+        self.assertIsNone(registry)
+
+    def test_get_attribute_registry_not_found(self):
+
+        self.sys_bios._attribute_registry = "Unknown"
+        registry = self.sys_bios.get_attribute_registry()
+        self.assertIsNone(registry)
